//////////////////////////////////////////////////////////
// Lampka bsw v.1.0
//////////////////////////////////////////////////////////

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

//////////////////////////////////////////////////////////
// Definicje portów
//////////////////////////////////////////////////////////

// Definicje wyprowadzeń portu B
#define LED 4
#define BTN 1
#define FON 2
#define FR  3

//Definicje trybów
#define MODE_OFF 0
#define MODE_ON 1
#define MODE_BLINK 2

#define ON_TIME 544
#define OFF_TIME 816
#define DARK_LEVEL 30 

//Obsługa wyprowadzeń 
#define LED_ON  PORTB |= _BV(LED)
#define LED_OFF PORTB &= ~(_BV(LED))

#define FR_ON  PORTB |= _BV(FON)
#define FR_OFF PORTB &= ~(_BV(FON))

//////////////////////////////////////////////////////////
// GLOBAL
//////////////////////////////////////////////////////////

volatile uint8_t g_Mode = 0; //0 OFF 1 ON  2 BLINK
volatile uint16_t g_RunTimer = 0;
volatile uint16_t g_DarkTimer = 0;

//////////////////////////////////////////////////////////
// MAIN
//////////////////////////////////////////////////////////

int main(void)
{
   //konfiguracja portów - wyjść
   DDRB = _BV(LED) | _BV(FON);

   //wyłączenie komparatora analogowego (oszczędność prądu)
   ACSR = _BV(ACD);

   //ustawienie timera 0
   TIMSK0 = _BV(OCIE0A);
   TCCR0B = _BV(WGM02);
   TCCR0B |= _BV(CS00) | _BV(CS02);

   //włączenie obsługi przerwania INT0
   GIMSK = _BV(INT0);   //External Interrupt Request 0 Enable

   //Any logical change on INT0 generates an interrupt request.
   //MCUCR = _BV(ISC00); 
 
   //umożliwienie usypiania
   MCUCR |= _BV(SE);

   //Konfiguracja przetwornika A/C
   ADMUX |= _BV(ADLAR);   //pomiar 8 bitowy
   ADMUX |= _BV(MUX1) | _BV(MUX0);    //przetwornik na porcie ADC3 (PB3)
                          //napięcie odniesienia = napięcie zasialania
   ADCSRA |= _BV(ADPS1)|_BV(ADPS0);  //prescaler/8
   ADCSRA |= _BV(ADIE);   //Wygeneruj przerwanie na koniec przetwarzania
   ADCSRA |= _BV(ADSC);   //Start konwersji
   ADCSRA |= _BV(ADEN);   //Włącz przetwornik

   //stan portów
   PORTB = _BV(BTN);

   //globalne zezwolenie na przerwania
   sei();

   for(;;)
   {	  
	  if(g_Mode == MODE_BLINK && g_DarkTimer > ON_TIME) 
	  {
		  g_Mode = MODE_ON;
		  g_RunTimer = 0;   
		  LED_OFF;
	  }	  
	   	   
	  if(g_RunTimer > OFF_TIME) 
	  {
		  g_Mode = MODE_OFF;   
	      LED_OFF;
	      FR_OFF;
          //usypianie
          set_sleep_mode(SLEEP_MODE_PWR_DOWN);
          sleep_mode();
      }
   }
   return 0;
}

//////////////////////////////////////////////////////////
// SIGNAL
//////////////////////////////////////////////////////////

//BTN
ISR(INT0_vect)
{
   if(g_Mode == MODE_OFF)
   {
	  g_Mode = MODE_ON;
      FR_ON;
   }	
   g_RunTimer = 0;	 
} 

//ADC
ISR(ADC_vect)
{
   if(ADCH>DARK_LEVEL)
   {
	  g_Mode = MODE_BLINK;
      g_DarkTimer = 0;
   }
   ADCSRA |= _BV(ADSC);
}

//TIMER
ISR(TIM0_COMPA_vect)
{
   if(g_Mode == MODE_BLINK) 
   {
      PORTB ^= _BV(LED);
      g_DarkTimer++;
   }         
   g_RunTimer++;
}	

//////////////////////////////////////////////////////////
// END
//////////////////////////////////////////////////////////
